import 'package:flutter/material.dart';
import 'package:nav_stack/nav_stack.dart';

import 'advanced_tabs_demo_pages.dart';
import 'advanced_tabs_demo_scaffold.dart';

class AdvancedTabsDemo extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp.router(
        routeInformationParser: NavStackParser(),
        routerDelegate: NavStackDelegate(onGenerateStack: (context, nav) {
          void _handleComposePressed() => nav.path = ComposePage.path;
          return PathStack(
            routes: {
              [AppPaths.tabsCategory]: PathStack(
                // Main scaffold is wrapped here
                scaffoldBuilder: (_, stack) => MainScaffold(
                  child: stack,
                  currentPath: NavStack.of(context).path,
                  // Goto /compose when this is pressed
                  onComposePressed: _handleComposePressed,
                ),
                transitionBuilder: (_, stack, animation) => FadeTransition(opacity: animation, child: stack),
                routes: {
                  // Home
                  [HomePage.path]: HomePage("").toStackRoute(),
                  // Settings
                  [AppPaths.settings]: PathStack(
                    // Settings scaffold is wrapped here
                    scaffoldBuilder: (_, child) => SettingsScaffold(child: child),
                    routes: {
                      [ProfileSettings.path]: ProfileSettings("").toStackRoute(),
                      [AlertSettings.path]: AlertSettings("").toStackRoute(),
                      [BillingSettings.path]: BillingSettings("").toStackRoute(maintainState: false),
                    },
                  ).toStackRoute(),
                  // Inbox
                  [AppPaths.inbox]: PathStack(
                    scaffoldBuilder: (_, child) => InboxScaffold(child: child),
                    routes: {
                      [InboxPage.friendsPath]: InboxPage(InboxType.friends).toStackRoute(),
                      [InboxPage.unreadPath]: InboxPage(InboxType.unread).toStackRoute(),
                      [InboxPage.archivedPath]: InboxPage(InboxType.archived).toStackRoute(),
                    },
                  ).toStackRoute(),
                },
              ).toStackRoute(),

              /// Non stateful full-screen route
              [ComposePage.path]: ComposePage().toStackRoute(maintainState: false),

              /// Inject itemId param into detailsView
              [DetailsPage.path + ":id"]: StackRouteBuilder(
                maintainState: false,
                builder: (_, args) => DetailsPage(itemId: args["id"]),
              )
            },
          );
        }));
  }
}
