import 'dart:async';

import 'package:flutter/material.dart';
import 'package:nav_stack/nav_stack.dart';

/// Change this value to test the route guard. When false, you should not be able to access the /in routes.
bool isConnected = true;

class BasicDemo extends StatelessWidget {
  @override
  Widget build(BuildContext _) {
    return MaterialApp.router(
        routeInformationParser: NavStackParser(),
        routerDelegate: NavStackDelegate(onGenerateStack: (context, nav) {
          return PathStack(
            scaffoldBuilder: (_, stack) => _MyScaffold(stack),
            routes: {
              ["/login", "/"]: LoginScreen().toStackRoute(),
              ["/in/"]: PathStack(
                routes: {
                  ["profile/:profileId"]: StackRouteBuilder(
                    builder: (_, args) => ProfileScreen(profileId: args["profileId"] ?? ""),
                  ),
                  ["settings"]: SettingsScreen().toStackRoute(),
                },
              ).toStackRoute(onBeforeEnter: (_) {
                if (!isConnected) nav.redirect("/login", onComplete: () => showAuthWarning(context));
                return isConnected; // If we return false, the route will not be entered.
              }),
            },
          );
        }));
  }
}

void showAuthWarning(BuildContext context) {
  showDialog(context: context, builder: (_) => AuthErrorDialog());
}

class AuthErrorDialog extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Center(
        child: GestureDetector(
            onTap: () => Navigator.of(context).pop(), // Pop still works for dialogs!
            child: Container(width: 400, height: 300, color: Colors.purple, child: buildText("No soup for you!!"))));
  }
}

class _MyScaffold extends StatelessWidget {
  _MyScaffold(this.child);
  final Widget child;
  @override
  Widget build(BuildContext context) {
    final navStack = NavStack.of(context);
    Widget buildBtn(String value) =>
        Expanded(child: TextButton(child: Text(value), onPressed: () => navStack.path = value));
    return Scaffold(
      body: Column(
        children: [
          Row(children: [buildBtn("/in/profile/99"), buildBtn("/in/settings")]),
          Flexible(child: child),
        ],
      ),
    );
  }
}

class HomeScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) => Center(child: Text("HomeScreen"));
}

class LoginScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) => Center(child: Text("LoginScreen"));
}

class MessagesScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) => Center(child: Text("MessagesScreen"));
}

class ProfileScreen extends StatelessWidget {
  const ProfileScreen({Key? key, this.profileId = ""}) : super(key: key);
  final String profileId;

  @override
  Widget build(BuildContext context) => Center(child: Text("ProfileScreen"));
}

class SettingsScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) => Center(child: Text("SettingsScreen"));
}

Text buildText(String value) => Text(value, style: TextStyle(fontSize: 32));
