library nav_stack;

import 'dart:async';
import 'package:flutter/material.dart';
import 'package:path_stack/path_stack.dart';

export 'package:path_stack/path_stack.dart';
export 'routing.dart';

part "inherited_nav_stack_controller.dart";

/*
TODO:
  * Build a demo that does some sort of dynamic user-flow:
      [/signup/:step(d+)] : builder(_, args) => SignupFlowPage(step: args['steps'])
  * Build some example that can update
  * Test popUntil, etc
  * Test dialogs inside of StackRouteBuilder
  * Add hook to handle custom BackBtnPressed handler
  * Redirect handler should be able to redirect on the fly... but there might be an issue with recursion? Theoretically redirect chains could go forever...
    * Simple enough to test... what should happen here? Bail after some max number of redirects?
  * NavStack could have a true onBeforeChange handler, as oppossed to PathStack which is coupled directly to .path
  * Create some sort of AppPath.builder method? Or many methods???
 */

class NavStackController with ChangeNotifier {
  NavStackController({this.onRouteChanging, this.initialPath, required this.stackBuilder});

  /// Called whenever the current path has changed
  final void Function(String value)? onRouteChanging;

  /// Must return a PathStack widget
  final PathStack Function(BuildContext context, NavStackController controller) stackBuilder;

  /// Determines what the initial path should be (if the OS has not provided one)
  final String? initialPath;

  static NavStackController of(BuildContext context) =>
      (context.dependOnInheritedWidgetOfExactType<InheritedNavStackController>() as InheritedNavStackController)
          .controller;

  String _path = "/";
  String get path => _path;
  set path(String value) {
    if (value != _path) {
      onRouteChanging?.call(value);
    }
    _path = value;
    notifyListeners();
  }

  Map<String, String> _args = {};
  String get args => _path;

  List<String> history = [];

  /// Steps back one level in the NavStack history
  bool goBack() {
    if (history.length > 1) {
      String prevPath = history[history.length - 2];
      history..removeLast()..removeLast(); // remove last 2 history entries
      path = prevPath; // switch, adding a new history entry
      return true;
    }
    return false;
  }

  /// Go back in history and grab the first route that does not match the provided route.
  /// Used in this demo to close all of the details pages you might have in your history stack
  void popMatching(String value, {bool exactMatch = false}) {
    int index = history.lastIndexWhere((element) => exactMatch ? element != value : element.contains(value) == false);
    List<String> newHistory = List.from(history)..removeRange(index + 1, history.length);
    path = newHistory.last;
    //TODO: Is it an issue that routeChanged handlers will fire before history stack is finalized? Would it be better to have something like ignoreNext?
    history = newHistory;
  }

  /// Pop all pages until we find a match
  void popUntil(String value, {bool exactMatch = false}) {
    int index = history.lastIndexWhere((element) => exactMatch ? element == value : element.contains(value));
    List<String> newHistory = List.from(history)..removeRange(index + 1, history.length);
    path = newHistory.last;
    history = newHistory;
  }

  /// Changes path without creating a new entry stack
  void replacePath(String value) {
    history.removeLast();
    path = value;
  }

  /// Removes all history except the current
  void clearHistory() => history = List.from([history.last]);

  /// Meant to be used from with [onBeforeUpdate] inside of [StackRouteBuilder],
  /// Changes path in the next frame to avoid errors when building mid-build.
  void redirect(String path, {VoidCallback? onComplete}) async => scheduleMicrotask(() {
        this.path = path;
        onComplete?.call();
      });
}
